import React from 'react'
import { mount } from 'enzyme'
import { mockMonthlyForecastJanuaryTestProfile1 } from '../../../tests/__mocks__/profileType.mock'
import { FluidType } from 'enum/fluid.enum'
import { MonthlyForecast } from 'models'
import AnalysisConsumptionRow from './AnalysisConsumptionRow'

jest.mock('cozy-ui/transpiled/react/I18n', () => {
  return {
    useI18n: jest.fn(() => {
      return {
        t: (str: string) => str,
      }
    }),
  }
})

describe('AnalysisConsumptionRow component', () => {
  const userPriceConsumption = 20
  const homePriceConsumption = 18
  const performanceValue: number | null = 25
  const forecast: MonthlyForecast = mockMonthlyForecastJanuaryTestProfile1
  const connected = true
  const noData = false

  it('should be rendered correctly for Multifluid and at least fluid connected', async () => {
    const mockPerformanceValue: number | null = null
    const wrapper = mount(
      <AnalysisConsumptionRow
        fluid={FluidType.MULTIFLUID}
        userPriceConsumption={userPriceConsumption}
        homePriceConsumption={homePriceConsumption}
        performanceValue={mockPerformanceValue}
        forecast={forecast}
        connected={connected}
        noData={noData}
      />
    )
    expect(wrapper.find('.consumption-multifluid').exists()).toBeTruthy()
    expect(wrapper.find('.price').first().text()).toBe('20,00 €')
    expect(wrapper.find('.price').last().text()).toBe('18,00 €')
    expect(wrapper.find('.graph').exists()).toBeTruthy()
    expect(wrapper.find('.not-connected').exists()).toBeFalsy()
  })

  it('should be rendered correctly for Multifluid and at none fluid connected', async () => {
    const mockPerformanceValue: number | null = null
    const mockConnected = false
    const wrapper = mount(
      <AnalysisConsumptionRow
        fluid={FluidType.MULTIFLUID}
        userPriceConsumption={userPriceConsumption}
        homePriceConsumption={homePriceConsumption}
        performanceValue={mockPerformanceValue}
        forecast={forecast}
        connected={mockConnected}
        noData={noData}
      />
    )
    expect(wrapper.find('.consumption-multifluid').exists()).toBeTruthy()
    expect(wrapper.find('.price').first().text()).toBe('analysis.not_connected')
    expect(wrapper.find('.price').last().text()).toBe('18,00 €')
    expect(wrapper.find('.graph').exists()).toBeFalsy()
    expect(wrapper.find('.not-connected').exists()).toBeTruthy()
  })

  it('should be rendered correctly for singleFluid connected for average', async () => {
    const wrapper = mount(
      <AnalysisConsumptionRow
        fluid={FluidType.ELECTRICITY}
        userPriceConsumption={userPriceConsumption}
        homePriceConsumption={homePriceConsumption}
        performanceValue={performanceValue}
        forecast={forecast}
        connected={connected}
        noData={noData}
      />
    )
    expect(wrapper.find('.consumption-electricity').exists()).toBeTruthy()
    expect(wrapper.find('.price').first().text()).toBe(
      '25 FLUID.ELECTRICITY.UNIT'
    )
    expect(wrapper.find('.price').last().text()).toBe(
      '4340 FLUID.ELECTRICITY.UNIT'
    )
    expect(wrapper.find('.graph').exists()).toBeTruthy()
    expect(wrapper.find('.not-connected').exists()).toBeFalsy()
  })

  it('should be rendered correctly for singleFluid not connected', async () => {
    const mockConnected = false
    const wrapper = mount(
      <AnalysisConsumptionRow
        fluid={FluidType.ELECTRICITY}
        userPriceConsumption={userPriceConsumption}
        homePriceConsumption={homePriceConsumption}
        performanceValue={performanceValue}
        forecast={forecast}
        connected={mockConnected}
        noData={noData}
      />
    )
    expect(wrapper.find('.consumption-electricity').exists()).toBeTruthy()
    expect(wrapper.find('.price').first().text()).toBe('analysis.not_connected')
    expect(wrapper.find('.price').last().text()).toBe(
      '4340 FLUID.ELECTRICITY.UNIT'
    )
    expect(wrapper.find('.graph').exists()).toBeFalsy()
    expect(wrapper.find('.not-connected').exists()).toBeTruthy()
  })

  it('should be rendered correctly for singleFluid with none performance value', async () => {
    const mockPerformanceValue: number | null = null
    const wrapper = mount(
      <AnalysisConsumptionRow
        fluid={FluidType.ELECTRICITY}
        userPriceConsumption={userPriceConsumption}
        homePriceConsumption={homePriceConsumption}
        performanceValue={mockPerformanceValue}
        forecast={forecast}
        connected={connected}
        noData={noData}
      />
    )
    expect(wrapper.find('.consumption-electricity').exists()).toBeTruthy()
    expect(wrapper.find('.price').first().text()).toBe('-')
    expect(wrapper.find('.price').last().text()).toBe(
      '4340 FLUID.ELECTRICITY.UNIT'
    )
    expect(wrapper.find('.graph').exists()).toBeTruthy()
    expect(wrapper.find('.not-connected').exists()).toBeFalsy()
  })

  it('should be rendered correctly with unit', async () => {
    const mockforecast: MonthlyForecast = {
      ...mockMonthlyForecastJanuaryTestProfile1,
      fluidForecast: [
        {
          detailsMonthlyForecast:
            mockMonthlyForecastJanuaryTestProfile1.fluidForecast[0]
              .detailsMonthlyForecast,
          fluidType: 0,
          load: 50,
          value: 25,
        },
      ],
    }
    const wrapper = mount(
      <AnalysisConsumptionRow
        fluid={FluidType.ELECTRICITY}
        userPriceConsumption={userPriceConsumption}
        homePriceConsumption={homePriceConsumption}
        performanceValue={performanceValue}
        forecast={mockforecast}
        connected={connected}
        noData={noData}
      />
    )
    expect(wrapper.find('.consumption-electricity').exists()).toBeTruthy()
    expect(wrapper.find('.price').first().text()).toBe(
      '25 FLUID.ELECTRICITY.UNIT'
    )
    expect(wrapper.find('.price').last().text()).toBe(
      '50 FLUID.ELECTRICITY.UNIT'
    )
    expect(wrapper.find('.graph').exists()).toBeTruthy()
    expect(wrapper.find('.not-connected').exists()).toBeFalsy()
  })
})
