import React, { useState, useEffect } from 'react'
import './analysisView.scss'
import { DateTime } from 'luxon'
import { useClient } from 'cozy-client'
import ConsumptionDataManager from 'services/consumption.service'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { DataloadValueDetail, TimePeriod } from 'models'
import PieChart from './PieChart'
import './totalAnalysisChart.scss'
import { getNavPicto } from 'utils/picto'
import Icon from 'cozy-ui/transpiled/react/Icon'

import { formatNumberValues } from 'utils/utils'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'

interface TotalAnalysisChartProps {
  analysisDate: DateTime
  fluidTypes: FluidType[]
}

const TotalAnalysisChart: React.FC<TotalAnalysisChartProps> = ({
  analysisDate,
  fluidTypes,
}: TotalAnalysisChartProps) => {
  const [dataLoadValueDetailArray, setDataLoadValueDetailArray] = useState<
    DataloadValueDetail[] | null
  >(null)
  const [totalLoadValue, setTotalLoadValue] = useState<number>(0)
  const client = useClient()
  const { t } = useI18n()
  const arcWidth = 30
  const radius = Math.min(375, innerWidth - 100)
  const outerRadius = radius / 2
  const innerRadius = outerRadius - arcWidth

  useEffect(() => {
    let subscribed = true
    async function getTotalData() {
      const timePeriod: TimePeriod = {
        startDate: analysisDate.minus({ month: 1 }).startOf('month'),
        endDate: analysisDate.minus({ month: 1 }).endOf('month'),
      }
      const consumptionService = new ConsumptionDataManager(client)
      const monthTotalData = await consumptionService.getGraphData(
        timePeriod,
        TimeStep.MONTH,
        fluidTypes,
        undefined,
        undefined,
        true
      )
      if (monthTotalData?.actualData) {
        setDataLoadValueDetailArray(monthTotalData.actualData[0].valueDetail)
        setTotalLoadValue(monthTotalData.actualData[0].value)
      }
    }
    if (subscribed) {
      getTotalData()
    }
    return () => {
      subscribed = false
    }
  }, [analysisDate, client, fluidTypes])
  return (
    <div
      className="totalAnalysis-container"
      style={{
        minHeight: radius + 100,
      }}
    >
      <div className="text-24-normal title">{t('analysis_pie.total')}</div>
      {dataLoadValueDetailArray && (
        <PieChart
          dataloadValueDetailArray={dataLoadValueDetailArray}
          totalValue={totalLoadValue}
          width={radius}
          height={radius}
          innerRadius={innerRadius}
          outerRadius={outerRadius}
          currentAnalysisDate={analysisDate
            .minus({ month: 1 })
            .startOf('month')}
        />
      )}
      {dataLoadValueDetailArray && fluidTypes.length > 1 && (
        <div className="total-card-container">
          {dataLoadValueDetailArray.map((dataload, index) => {
            return (
              <div key={index} className="total-card">
                <div className="text-18-bold fluidconso">
                  {dataload.value !== -1
                    ? `${formatNumberValues(dataload.value)} €`
                    : '--- €'}
                </div>
                <Icon
                  className="euro-fluid-icon"
                  icon={getNavPicto(index, true, true)}
                  size={38}
                />
                <div className="text-16-normal">
                  {t('FLUID.' + FluidType[index] + '.LABEL')}
                </div>
              </div>
            )
          })}
        </div>
      )}
    </div>
  )
}

export default TotalAnalysisChart
