import React, { useState, useEffect } from 'react'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { Account, AccountAuthData, FluidStatus } from 'models'
import './formLogin.scss'
import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'
import Button from '@material-ui/core/Button'
import TrailingIcon from 'assets/icons/ico/trailing-icon.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import { FluidType } from 'enum/fluid.enum'
import { getPartnerPicto } from 'utils/picto'
import useKonnectorAuth from 'components/Hooks/useKonnectorAuth'
import * as Sentry from '@sentry/react'

interface FormLoginProps {
  fluidStatus: FluidStatus
  highlightedStyle?: boolean
}

const FormLogin: React.FC<FormLoginProps> = ({
  fluidStatus,
  highlightedStyle = true,
}: FormLoginProps) => {
  const { t } = useI18n()
  const konnectorSlug: string = fluidStatus.connection.konnectorConfig.slug
  const lastKnownCredentials: string | undefined =
    fluidStatus.connection.konnectorConfig.lastKnownCredentials
  const fluidName: string = FluidType[fluidStatus.fluidType]
  const account: Account | null = fluidStatus.connection.account

  const [login, setLogin] = useState<string>('')
  const [password, setPassword] = useState<string>('')
  const [error, setError] = useState<string>('')
  const [loading, setLoading] = useState<boolean>(false)

  const icon = getPartnerPicto(konnectorSlug, highlightedStyle)
  const [connect, update, connectError] = useKonnectorAuth(
    fluidStatus,
    login,
    password
  )

  const changeLogin = (value: string) => {
    if ((/[0-9]/.test(value) && value.length <= 7) || value === '') {
      setError('')
      setLogin(value)
    }
  }

  const changePassword = (value: string) => {
    setError('')
    setPassword(value)
  }

  const revealPassword = (idInput: string) => {
    const input = document.getElementById(idInput)
    if (input) {
      if (input.getAttribute('type') === 'password') {
        input.setAttribute('type', 'text')
      } else {
        input.setAttribute('type', 'password')
      }
    }
  }

  const handleSubmit = async (e: React.FormEvent<HTMLFormElement>) => {
    e.preventDefault()
    try {
      setError('')
      setLoading(true)
      if (!login || !password) {
        setError(t('konnector_form.error_no_login_password'))
        setLoading(false)
        return null
      }
      if (!account) {
        await connect()
        setLoading(false)
      } else {
        await update()
      }
    } catch (error) {
      Sentry.captureException(JSON.stringify({ error }))
      setLoading(false)
    }
  }

  useEffect(() => {
    if (connectError) setError(connectError)
  }, [connectError])

  useEffect(() => {
    if (account?.auth) {
      const auth = account.auth
      const authData = auth as AccountAuthData
      if (authData.login) {
        setLogin(authData.login)
      }
    } else if (lastKnownCredentials) {
      setLogin(lastKnownCredentials)
      setError(t('konnector_form.error_login_failed'))
    }
  }, [account, lastKnownCredentials, t])

  return (
    <form
      className="form"
      onSubmit={(e: React.FormEvent<HTMLFormElement>) => handleSubmit(e)}
    >
      <div className="form-group">
        <input
          id={'idFieldLogin' + fluidName}
          type="number"
          className={
            error === ''
              ? 'form-control form-input'
              : 'form-control form-input --error'
          }
          aria-label={t('auth.' + konnectorSlug + '.connect.login')}
          placeholder={t('auth.' + konnectorSlug + '.connect.login')}
          name="login"
          onChange={(e: React.ChangeEvent<HTMLInputElement>) =>
            changeLogin(e.target.value)
          }
          value={login}
          inputMode="numeric"
        ></input>
      </div>
      <div className="form-group">
        <input
          id={'idFieldPassword' + fluidName}
          type="password"
          className={
            error === ''
              ? 'form-control form-input'
              : 'form-control form-input --error'
          }
          aria-label={t('auth.' + konnectorSlug + '.connect.password')}
          placeholder={t('auth.' + konnectorSlug + '.connect.password')}
          name="password"
          onChange={(e: React.ChangeEvent<HTMLInputElement>) =>
            changePassword(e.target.value)
          }
          value={password}
        />
        <span>
          <StyledIconButton
            icon={TrailingIcon}
            aria-label={t('auth.accessibility.button_reveal_password')}
            className="form-trailing-icon"
            sized={22}
            onClick={() => revealPassword('idFieldPassword' + fluidName)}
          />
        </span>
      </div>
      <div className="form-message">{error === '' ? null : error}</div>
      <div className="kloginauthform">
        <div className="kloginauthform-button">
          <Button
            type="submit"
            aria-label={t('auth.accessibility.button_connect')}
            disabled={loading}
            classes={{
              root: `${
                highlightedStyle ? 'btn-highlight' : 'btn-secondary-negative'
              }`,
              label: 'text-18-bold',
            }}
          >
            <div className="authform-button-content">
              <div className="authform-button-content-icon">
                {icon && <StyledIcon icon={icon} size={80} />}
              </div>
              <div className="authform-button-text text-18-bold">
                <div>{t('auth.' + konnectorSlug + '.connect.label1')}</div>
                <div>{t('auth.' + konnectorSlug + '.connect.label2')}</div>
              </div>
            </div>
          </Button>
        </div>
      </div>
    </form>
  )
}

export default FormLogin
