import Button from '@material-ui/core/Button'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { OAuthWindow } from 'cozy-harvest-lib/dist/components/OAuthWindow'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { UsageEventType } from 'enum/usageEvent.enum'
import { FluidStatus, Konnector } from 'models'
import React, { useCallback, useEffect, useState } from 'react'
import { useDispatch, useSelector } from 'react-redux'
import UsageEventService from 'services/usageEvent.service'
import { AppStore } from 'store'
import { setShouldRefreshConsent } from 'store/global/global.actions'
import { getPartnerPicto } from 'utils/picto'
import './auth.scss'

interface FormOAuthProps {
  konnector: Konnector | null
  onSuccess: Function
  highlightedStyle?: boolean
  fluidStatus: FluidStatus
}

const FormOAuth: React.FC<FormOAuthProps> = ({
  konnector,
  onSuccess,
  highlightedStyle = true,
  fluidStatus,
}: FormOAuthProps) => {
  const IDLE = 'idle'
  const WAITING = 'waiting'

  const { t } = useI18n()
  const client = useClient()
  const [status, setStatus] = useState<string>(IDLE)
  const { shouldRefreshConsent } = useSelector(
    (state: AppStore) => state.ecolyo.global
  )
  const dispatch = useDispatch()
  const endOAuth = useCallback(() => {
    setStatus(IDLE)
    //Set back to false the variable that allows to automatically refresh the consent (deletes and recreates the account)
    dispatch(setShouldRefreshConsent(false))
  }, [dispatch])

  const startOAuth = useCallback(async () => {
    // If first connexion, send the usage event
    if (konnector?.slug && fluidStatus.lastDataDate === null) {
      await UsageEventService.addEvent(client, {
        type: UsageEventType.KONNECTOR_ATTEMPT_EVENT,
        target: konnector.slug,
        result: 'error',
      })
    }
    setStatus(WAITING)
  }, [client, fluidStatus.lastDataDate, konnector])

  const handleAccountId = useCallback(
    (accountId: string) => {
      endOAuth()
      onSuccess(accountId)
    },
    [endOAuth, onSuccess]
  )

  const handleOAuthCancel = useCallback(() => {
    endOAuth()
  }, [endOAuth])

  const icon = getPartnerPicto(
    konnector ? konnector.slug : '',
    highlightedStyle
  )
  const isWaiting = status === WAITING

  useEffect(() => {
    if (shouldRefreshConsent) {
      // If user has selected accept button on Expired consent modal, his account has been deleted on KonnectorViewerCard, such as his consent (for GRDF), then automatically launch oauth connxion flow
      startOAuth()
    }
  }, [shouldRefreshConsent, startOAuth])

  if (!konnector) {
    return null
  }

  return (
    <>
      <Button
        aria-label={t('auth.accessibility.button_connect')}
        onClick={startOAuth}
        disabled={isWaiting}
        classes={{
          root: `${
            highlightedStyle ? 'btn-highlight' : 'btn-secondary-negative'
          }`,
          label: 'text-18-bold',
        }}
      >
        <div className="authform-button-content">
          <div className="authform-button-content-icon">
            {isWaiting ? (
              <Loader color="black" />
            ) : (
              <StyledIcon icon={icon} size={80} />
            )}
          </div>
          <div className="authform-button-text text-18-bold">
            <div>{t('auth.' + konnector.slug + '.connect.label1')}</div>
            <div>{t('auth.' + konnector.slug + '.connect.label2')}</div>
          </div>
        </div>
      </Button>
      {isWaiting && (
        <OAuthWindow
          client={client}
          konnector={konnector}
          redirectSlug={client.appMetadata.slug}
          onSuccess={handleAccountId}
          onCancel={handleOAuthCancel}
          t={t}
        />
      )}
    </>
  )
}

export default FormOAuth
