import React, { useState, useEffect } from 'react'
import './ecogestureModal.scss'
import classNames from 'classnames'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { AppStore } from 'store'
import { useSelector } from 'react-redux'
import { getPicto } from 'utils/picto'
import useExploration from 'components/Hooks/useExploration'
import { importIconbyId } from 'utils/utils'

import { Ecogesture } from 'models'

import Icon from 'cozy-ui/transpiled/react/Icon'
import Dialog from '@material-ui/core/Dialog'
import IconButton from '@material-ui/core/IconButton'
import Button from '@material-ui/core/Button'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import EfficientyRating from './EfficientyRating'
import CloseIcon from 'assets/icons/ico/close.svg'
import defaultIcon from 'assets/icons/visu/ecogesture/default.svg'

interface EcogestureModalProps {
  open: boolean
  ecogesture: Ecogesture
  isAction: boolean
  handleCloseClick: () => void
  selectEcogesture?: () => void
}

const EcogestureModal: React.FC<EcogestureModalProps> = ({
  open,
  ecogesture,
  isAction,
  handleCloseClick,
  selectEcogesture,
}: EcogestureModalProps) => {
  const { t } = useI18n()
  const [ecogestureIcon, setEcogestureIcon] = useState('')
  const [isMoreDetail, setIsMoreDetail] = useState(false)
  const { currentChallenge } = useSelector(
    (state: AppStore) => state.ecolyo.challenge
  )

  const [, setValidExploration] = useExploration()
  const toggleMoreDetail = () => {
    setIsMoreDetail(prev => !prev)
  }
  useEffect(() => {
    async function handleEcogestureIcon() {
      const icon = await importIconbyId(ecogesture.id, 'ecogesture')
      if (icon) {
        setEcogestureIcon(icon)
      } else {
        setEcogestureIcon(defaultIcon)
      }
      if (currentChallenge?.exploration.ecogesture_id === ecogesture._id) {
        setValidExploration(currentChallenge.exploration.id)
      }
    }
    if (ecogesture) {
      handleEcogestureIcon()
    }
  }, [ecogesture, setValidExploration, currentChallenge])

  return (
    <Dialog
      open={open}
      onClose={handleCloseClick}
      aria-labelledby={'accessibility-title'}
      classes={{
        root: 'modal-root',
        paper: 'modal-paper no-padding blue-border',
      }}
    >
      <div id={'accessibility-title'}>
        {isAction
          ? t('ecogesture_modal.accessibility.window_title_action')
          : t('ecogesture_modal.accessibility.window_title_ecogesture')}
      </div>
      <IconButton
        aria-label={t('ecogesture_modal.accessibility.button_close')}
        className="modal-paper-close-button"
        onClick={handleCloseClick}
      >
        <Icon icon={CloseIcon} size={16} />
      </IconButton>
      <div className="em-header text-14-normal-uppercase">
        {isAction
          ? t('ecogesture_modal.title_action')
          : t('ecogesture_modal.title_ecogesture')}
      </div>

      <div className="em-root ecogesture-modal">
        <div className="em-content">
          <div className="em-content-box-img">
            {ecogestureIcon && (
              <StyledIcon className="icon" icon={ecogestureIcon} size={140} />
            )}
          </div>
          <div className="em-title text-24-bold ">{ecogesture.shortName}</div>
          <div className="em-detail">
            <div className="em-detail-efficiency">
              <span className="em-efficiency">
                {t('ecogesture_modal.efficiency')}
              </span>
              <EfficientyRating result={Math.round(ecogesture.efficiency)} />
            </div>
            <div className="em-picto-flow">
              {ecogesture.fluidTypes.map((fluid, index) => (
                <div key={index}>
                  <StyledIcon
                    className="em-pic-content"
                    icon={getPicto(fluid)}
                    size={25}
                  />
                </div>
              ))}
            </div>
          </div>
          <div className="text-16-bold long-name">{ecogesture.longName}</div>
          {isAction ? (
            <Button
              aria-label={t(
                'ecogesture_modal.accessibility.button_select_action'
              )}
              classes={{
                root: 'btn-action-launch',
                label: 'text-16-bold',
              }}
              onClick={selectEcogesture}
            >
              {t('ecogesture_modal.select_action')}
            </Button>
          ) : (
            <>
              <div
                className={classNames('em-description text-16-normal-150', {
                  ['block']: isMoreDetail === true,
                })}
              >
                {ecogesture.longDescription}
              </div>
              <Button
                aria-label={t(
                  'ecogesture_modal.accessibility.button_see_more_detail'
                )}
                onClick={toggleMoreDetail}
                classes={{
                  root: 'btn-secondary-negative',
                  label: 'text-14-normal',
                }}
              >
                {isMoreDetail
                  ? t('ecogesture_modal.show_less')
                  : t('ecogesture_modal.show_more')}
              </Button>
            </>
          )}
        </div>
      </div>
    </Dialog>
  )
}

export default EcogestureModal
