import React, { useCallback } from 'react'
import { useHistory } from 'react-router-dom'
import './explorationOngoing.scss'
import { Client, useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { useDispatch } from 'react-redux'

import { UserChallenge } from 'models'
import { UserChallengeUpdateFlag } from 'enum/userChallenge.enum'
import Button from '@material-ui/core/Button'
import StarsContainer from 'components/Challenge/StarsContainer'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import {
  UserExplorationState,
  UserExplorationType,
} from 'enum/userExploration.enum'
import ChallengeService from 'services/challenge.service'
import { updateUserChallengeList } from 'store/challenge/challenge.actions'
import explorationIcon from 'assets/icons/visu/exploration/shield.svg'
import UsageEventService from 'services/usageEvent.service'
import { UsageEventType } from 'enum/usageEvent.enum'

interface ExplorationOngoingProps {
  userChallenge: UserChallenge
}

const ExplorationOngoing: React.FC<ExplorationOngoingProps> = ({
  userChallenge,
}: ExplorationOngoingProps) => {
  const client: Client = useClient()
  const { t } = useI18n()
  const dispatch = useDispatch()
  const history = useHistory()
  const goBack = useCallback(() => {
    history.goBack()
  }, [history])
  const startExploration = async () => {
    if (userChallenge.exploration.state !== UserExplorationState.ONGOING) {
      const challengeService = new ChallengeService(client)
      const updatedChallenge = await challengeService.updateUserChallenge(
        userChallenge,
        UserChallengeUpdateFlag.EXPLORATION_START
      )
      dispatch(updateUserChallengeList(updatedChallenge))
    }
    history.push('/challenges')
  }

  const validExploration = async () => {
    const challengeService = new ChallengeService(client)
    const updatedChallenge = await challengeService.updateUserChallenge(
      userChallenge,
      UserChallengeUpdateFlag.EXPLORATION_DONE
    )
    await UsageEventService.addEvent(client, {
      type: UsageEventType.EXPLORATION_END_EVENT,
      target: userChallenge.exploration.id,
      context: userChallenge.id,
      startDate: userChallenge.exploration.date,
    })
    dispatch(updateUserChallengeList(updatedChallenge))
    history.push('/challenges')
  }
  const renderButton = () => {
    switch (userChallenge.exploration.type) {
      case UserExplorationType.DECLARATIVE:
        return (
          <div className="button-start">
            <Button
              aria-label={t('exploration.accessibility.button_already_done')}
              onClick={validExploration}
              classes={{
                root: 'btn-secondary-negative',
                label: 'text-16-normal',
              }}
            >
              {t('exploration.button_already_done')}
            </Button>
            <Button
              aria-label={t('exploration.accessibility.button_already_done')}
              onClick={goBack}
              classes={{
                root: 'btn-secondary-negative',
                label: 'text-16-normal',
              }}
            >
              {t('exploration.button_come_back')}
            </Button>
          </div>
        )
      case UserExplorationType.ACTION:
      case UserExplorationType.ECOGESTURE:
      case UserExplorationType.CONSUMPTION:
        return (
          <div className="button-start">
            <Button
              aria-label={t('exploration.accessibility.button_start')}
              onClick={startExploration}
              classes={{
                root: 'btn-secondary-negative',
                label: 'text-16-normal',
              }}
            >
              {t('exploration.button_start')}
            </Button>
          </div>
        )
    }
  }

  return (
    <>
      <div className="exploration-container">
        <div className="exploration-begin-container">
          <StyledIcon
            className="exploration-icon"
            icon={explorationIcon}
            size={180}
          />
          <StarsContainer
            result={userChallenge.progress.explorationProgress}
            isQuizBegin={true}
          />
          <div className="exploration-explanation text-18-bold">
            <div>{userChallenge.exploration.description}</div>
            <div>{userChallenge.exploration.complementary_description}</div>
          </div>
          {renderButton()}
        </div>
      </div>
    </>
  )
}

export default ExplorationOngoing
