import React, { useState, useEffect, useRef } from 'react'
import './fluidChartSwipe.scss'
import { useDispatch, useSelector } from 'react-redux'
import { AppStore } from 'store'
import { setCurrentIndex, setSelectedDate } from 'store/chart/chart.actions'
import SwipeableViews from 'react-swipeable-views'
import { virtualize } from 'react-swipeable-views-utils'
import { DateTime } from 'luxon'

import { FluidType } from 'enum/fluid.enum'
import FluidChartSlide from 'components/FluidChart/FluidChartSlide'
import DateChartService from 'services/dateChart.service'

const VirtualizeSwipeableViews = virtualize(SwipeableViews)

interface FluidChartSwipeProps {
  fluidType: FluidType
  showCompare: boolean
  setActive: React.Dispatch<React.SetStateAction<boolean>>
}

const FluidChartSwipe: React.FC<FluidChartSwipeProps> = ({
  fluidType,
  showCompare,
  setActive,
}: FluidChartSwipeProps) => {
  const dispatch = useDispatch()
  const { currentIndex, currentTimeStep, selectedDate, loading } = useSelector(
    (state: AppStore) => state.ecolyo.chart
  )
  const swipe = useRef<HTMLDivElement>(null)
  const [width, setWidth] = useState(0)
  const [height, setHeight] = useState(0)
  const [isSwitching, setIsSwitching] = useState(false)

  const handleChangeIndex = (index: number) => {
    const dateChartService = new DateChartService()
    let increment = 0
    if (currentIndex < index) {
      increment = dateChartService.defineIncrementForPreviousIndex(
        currentTimeStep,
        selectedDate,
        currentIndex
      )
    } else {
      increment = dateChartService.defineIncrementForNextIndex(
        currentTimeStep,
        selectedDate,
        currentIndex
      )
    }
    const updatedDate: DateTime = dateChartService.incrementeDate(
      currentTimeStep,
      selectedDate,
      increment
    )
    const updatedIndex: number = dateChartService.defineDateIndex(
      currentTimeStep,
      updatedDate
    )
    dispatch(setSelectedDate(updatedDate))
    dispatch(setCurrentIndex(updatedIndex))
  }

  useEffect(() => {
    function handleResize() {
      if (!loading) {
        const maxWidth = 940
        const maxHeight = 300
        const _width = swipe.current
          ? swipe.current.offsetWidth > maxWidth
            ? maxWidth
            : swipe.current.offsetWidth
          : 400
        setWidth(_width)
        const _height = swipe.current
          ? swipe.current.offsetHeight > maxHeight
            ? maxHeight
            : swipe.current.offsetHeight
          : 300
        setHeight(_height)
      }
    }
    handleResize()
    window.addEventListener('resize', handleResize)
    return () => window.removeEventListener('resize', handleResize)
  }, [loading])

  useEffect(() => {
    function initIndex() {
      const dateChartService = new DateChartService()
      const updatedIndex: number = dateChartService.defineDateIndex(
        currentTimeStep,
        selectedDate
      )
      dispatch(setCurrentIndex(updatedIndex))
    }
    initIndex()
  }, [dispatch, currentTimeStep, selectedDate])

  return (
    <div className={'fluidchartswipe-root'} ref={swipe}>
      <VirtualizeSwipeableViews
        index={currentIndex}
        overscanSlideAfter={1}
        overscanSlideBefore={1}
        onChangeIndex={handleChangeIndex}
        slideRenderer={({ key, index }) => (
          <FluidChartSlide
            key={key}
            index={index}
            fluidType={fluidType}
            showCompare={showCompare}
            width={width}
            height={height}
            isSwitching={isSwitching}
            setActive={setActive}
          />
        )}
        enableMouseEvents
        onSwitching={!isSwitching ? () => setIsSwitching(true) : null}
        onTransitionEnd={() => {
          setIsSwitching(false)
        }}
        axis="x-reverse"
      />
    </div>
  )
}

export default FluidChartSwipe
