import Loader from 'components/Loader/Loader'
import { FluidState, FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { mount } from 'enzyme'
import { ChartState, FluidStatus, GlobalState } from 'models'
import React from 'react'
import * as reactRedux from 'react-redux'
import { Provider } from 'react-redux'
import { MockStoreEnhanced } from 'redux-mock-store'
import * as chartActions from 'store/chart/chart.actions'
import { mockTestProfile1 } from '../../../tests/__mocks__/profileType.mock'
import {
  createMockStore,
  mockExpiredElec,
  mockExpiredGas,
  mockInitialEcolyoState,
} from '../../../tests/__mocks__/store'
import ConsumptionView from './ConsumptionView'

jest.mock('cozy-ui/transpiled/react/I18n', () => {
  return {
    useI18n: jest.fn(() => {
      return {
        t: (str: string) => str,
      }
    }),
  }
})
const mockUpdateProfile = jest.fn()
jest.mock('services/profile.service', () => {
  return jest.fn(() => {
    return {
      updateProfile: mockUpdateProfile,
    }
  })
})

jest.mock('components/Header/CozyBar', () => 'mock-cozybar')
jest.mock('components/Header/Header', () => 'mock-header')
jest.mock('components/DateNavigator/DateNavigator', () => 'mock-datenavigator')
jest.mock('components/Content/Content', () => 'mock-content')
jest.mock('components/Home/FluidButtons', () => 'mock-fluidbuttons')
jest.mock('components/FluidChart/FluidChart', () => 'mock-fluidchart')
jest.mock('components/Home/ConsumptionDetails', () => 'mock-consumptiondetails')
jest.mock('components/Connection/Connection', () => 'mock-connection')
jest.mock(
  'components/Konnector/KonnectorViewerCard',
  () => 'mock-konnectorviewercard'
)
jest.mock(
  'components/PartnerIssue/PartnerIssueModal',
  () => 'mock-partnerissuemodal'
)
jest.mock('components/CustomPopup/CustomPopupModal', () => 'mock-custompopup')
jest.mock('components/Home/ReleaseNotesModal', () => 'mock-releasenotes')
jest.mock(
  'components/Connection/SGEConnect/SgeConnectView',
  () => 'mock-SgeConnectView'
)

const useSelectorSpy = jest.spyOn(reactRedux, 'useSelector')
const useDispatchSpy = jest.spyOn(reactRedux, 'useDispatch')
const setCurrentTimeStepSpy = jest.spyOn(chartActions, 'setCurrentTimeStep')

const mockedPartnersIssueModal = {
  enedis: false,
  egl: false,
  grdf: false,
}

describe('ConsumptionView component', () => {
  let store: MockStoreEnhanced<
    {
      ecolyo: {
        global: GlobalState
        chart?: ChartState
      }
    },
    {}
  >
  beforeEach(() => {
    store = createMockStore(mockInitialEcolyoState)
    useDispatchSpy.mockClear()
    setCurrentTimeStepSpy.mockClear()
  })

  it('should be rendered correctly', async () => {
    const mockFluidStatus: FluidStatus[] =
      mockInitialEcolyoState.global.fluidStatus
    mockFluidStatus[FluidType.ELECTRICITY].status = FluidState.DONE
    useSelectorSpy.mockReturnValue({
      chart: {
        currentTimeStep: TimeStep.WEEK,
        loading: true,
      },
      global: {
        fluidStatus: mockFluidStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('mock-cozybar').exists()).toBeTruthy()
    expect(wrapper.find('mock-header').exists()).toBeTruthy()
    expect(wrapper.find('mock-datenavigator').exists()).toBeTruthy()
    expect(wrapper.find('mock-fluidbuttons').exists()).toBeTruthy()
    expect(wrapper.find('mock-fluidchart').exists()).toBeTruthy()
    expect(wrapper.find('mock-consumptiondetails').exists()).toBeTruthy()
  })

  it('should display a spinner when fluid connected and is loading', () => {
    const mockFluidStatus: FluidStatus[] =
      mockInitialEcolyoState.global.fluidStatus
    mockFluidStatus[FluidType.ELECTRICITY].status = FluidState.DONE
    useSelectorSpy.mockReturnValue({
      chart: {
        currentTimeStep: TimeStep.WEEK,
        loading: true,
      },
      global: {
        fluidStatus: mockFluidStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('mock-cozybar').exists()).toBeTruthy()
    expect(wrapper.find('mock-header').exists()).toBeTruthy()
    expect(wrapper.find('mock-datenavigator').exists()).toBeTruthy()
    expect(wrapper.find(Loader).exists()).toBeTruthy()
  })

  it('should set CurrentTimeStep to WEEK when fluid != ELECTRICITY and timeStep = HALF_AN_HOUR', () => {
    useSelectorSpy.mockReturnValue({
      chart: {
        currentTimeStep: TimeStep.HALF_AN_HOUR,
        loading: true,
      },
      global: {
        fluidStatus: mockInitialEcolyoState.global.fluidStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
    })
    mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.GAS} />
      </Provider>
    )
    expect(setCurrentTimeStepSpy).toBeCalledTimes(1)
    expect(setCurrentTimeStepSpy).toHaveBeenCalledWith(TimeStep.WEEK)
  })

  it('should render konnector list when no fluid is connected', () => {
    useSelectorSpy.mockReturnValue({
      chart: {
        currentTimeStep: TimeStep.WEEK,
        loading: true,
      },
      global: {
        fluidStatus: [],
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      fluidStatus: [],
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.MULTIFLUID} />
      </Provider>
    )
    expect(wrapper.find('mock-consumptiondetails').exists()).toBeTruthy()
  })

  it('should render mutlifluid consumption if at least one fluid is connected', () => {
    const updatedStatus: FluidStatus[] =
      mockInitialEcolyoState.global.fluidStatus
    updatedStatus[1].status = FluidState.DONE
    useSelectorSpy.mockReturnValue({
      chart: {
        currentTimeStep: TimeStep.WEEK,
        loading: true,
      },
      global: {
        fluidStatus: updatedStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
      fluidStatus: updatedStatus,
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.MULTIFLUID} />
      </Provider>
    )
    expect(wrapper.find('.consumptionview-content').exists()).toBeTruthy()
  })

  it('should render Electricity when elec is connected', () => {
    const updatedStatus: FluidStatus[] =
      mockInitialEcolyoState.global.fluidStatus
    updatedStatus[0].status = FluidState.DONE
    useSelectorSpy.mockReturnValue({
      chart: {
        currentTimeStep: TimeStep.WEEK,
        loading: true,
      },
      global: {
        fluidStatus: updatedStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('.consumptionview-content').exists()).toBeTruthy()
    expect(wrapper.find('mock-consumptiondetails').exists()).toBeTruthy()
  })

  // todo describe and add multiple fluids ?
  it('should render partner issue Modal', async () => {
    const updatedStatus: FluidStatus[] =
      mockInitialEcolyoState.global.fluidStatus
    updatedStatus[0] = mockExpiredElec
    useSelectorSpy.mockReturnValue({
      chart: {
        currentTimeStep: TimeStep.WEEK,
        loading: true,
      },
      global: {
        fluidStatus: updatedStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
        openPartnersIssueModal: { ...mockedPartnersIssueModal, egl: true },
      },
    })
    useDispatchSpy.mockReturnValue(jest.fn())
    mockUpdateProfile.mockResolvedValue(mockTestProfile1)
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('mock-partnerissuemodal').exists()).toBeTruthy()
  })
  it('should show expired modal when a GRDF consent is expired', () => {
    const updatedStatus: FluidStatus[] =
      mockInitialEcolyoState.global.fluidStatus
    updatedStatus[0] = mockExpiredGas
    useSelectorSpy.mockReturnValue({
      chart: {
        currentTimeStep: TimeStep.WEEK,
        loading: true,
      },
      global: {
        fluidStatus: updatedStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
        openPartnersIssueModal: false,
      },
    })
    useDispatchSpy.mockReturnValue(jest.fn())
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.GAS} />
      </Provider>
    )
    expect(wrapper.find('.title').text()).toBe('consent_outdated.title.2')
  })
  it('should show expired modal when a Enedis consent is expired', () => {
    const updatedStatus: FluidStatus[] =
      mockInitialEcolyoState.global.fluidStatus
    updatedStatus[0] = mockExpiredElec
    useSelectorSpy.mockReturnValue({
      chart: {
        currentTimeStep: TimeStep.WEEK,
        loading: true,
      },
      global: {
        fluidStatus: updatedStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
        openPartnersIssueModal: mockedPartnersIssueModal,
      },
    })
    useDispatchSpy.mockReturnValue(jest.fn())
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )

    expect(wrapper.find('.title').text()).toBe('consent_outdated.title.0')
  })
  it('should render customPopup Modal', async () => {
    const updatedStatus: FluidStatus[] =
      mockInitialEcolyoState.global.fluidStatus
    updatedStatus[0] = mockExpiredElec
    useSelectorSpy.mockReturnValue({
      chart: {
        currentTimeStep: TimeStep.WEEK,
        loading: true,
      },
      global: {
        fluidStatus: updatedStatus,
        releaseNotes: mockInitialEcolyoState.global.releaseNotes,
        openPartnersIssueModal: mockedPartnersIssueModal,
      },
    })
    useDispatchSpy.mockReturnValue(jest.fn())
    mockUpdateProfile.mockResolvedValue(mockTestProfile1)
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('mock-custompopup').exists()).toBeTruthy()
  })
  it('should render releaseNotesModal if releaseNotes.show is true', async () => {
    const updatedStatus: FluidStatus[] =
      mockInitialEcolyoState.global.fluidStatus
    updatedStatus[0] = mockExpiredElec
    useSelectorSpy.mockReturnValue({
      chart: {
        currentTimeStep: TimeStep.WEEK,
        loading: true,
      },
      global: {
        fluidStatus: updatedStatus,
        openPartnersIssueModal: mockedPartnersIssueModal,
        releaseNotes: {
          show: true,
          notes: [{ description: 'description', title: 'title' }],
        },
      },
      openPartnersIssueModal: mockedPartnersIssueModal,
    })
    useDispatchSpy.mockReturnValue(jest.fn())
    mockUpdateProfile.mockResolvedValue(mockTestProfile1)
    const wrapper = mount(
      <Provider store={store}>
        <ConsumptionView fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('mock-releasenotes').exists()).toBeTruthy()
  })
})
