import React from 'react'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'

import { TimeStep } from 'enum/timeStep.enum'
import { PerformanceIndicator } from 'models'
import { formatNumberValues } from 'utils/utils'

import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import PileIcon from 'assets/icons/ico/coins.svg'

import GreenIndicatorIcon from 'assets/icons/visu/indicator/green.svg'
import RedIndicatorIcon from 'assets/icons/visu/indicator/red.svg'
import GreyIndicatorIcon from 'assets/icons/visu/indicator/grey.svg'
import ErrorIndicatorIcon from 'assets/icons/visu/indicator/error.svg'
import { FluidType } from 'enum/fluid.enum'
import './fluidPerformanceIndicator.scss'
import { convertDateToMonthString } from 'utils/date'
import { DateTime } from 'luxon'

interface PerformanceIndicatorContentProps {
  performanceIndicator: PerformanceIndicator
  timeStep: TimeStep
  fluidLackOfData?: Array<FluidType>
  analysisDate?: DateTime
}

const PerformanceIndicatorContent: React.FC<
  PerformanceIndicatorContentProps
> = ({
  performanceIndicator,
  fluidLackOfData = [],
  analysisDate,
}: PerformanceIndicatorContentProps) => {
  const { t } = useI18n()
  let displayedValue: string
  if (performanceIndicator?.value)
    displayedValue = formatNumberValues(performanceIndicator.value).toString()
  else displayedValue = '-----'

  let errorInPerf = false
  if (!performanceIndicator?.value) {
    errorInPerf = true
  }

  let perf: number | null = null
  let diffInEuro: number | null = null
  if (performanceIndicator?.value && performanceIndicator.compareValue) {
    perf =
      100 * (performanceIndicator.value / performanceIndicator.compareValue - 1)
    diffInEuro = performanceIndicator.value - performanceIndicator.compareValue
  }
  const perfString = perf ? formatNumberValues(perf) : ''
  const diffString = diffInEuro ? formatNumberValues(diffInEuro) : ''
  let perfStatus = ['error', ErrorIndicatorIcon]
  if (perf === null && !errorInPerf) perfStatus = ['nodata', ErrorIndicatorIcon]
  else if (perf === null && errorInPerf)
    perfStatus = ['error', ErrorIndicatorIcon]
  else if (perf && perf === 0) perfStatus = ['zero', GreyIndicatorIcon]
  else if (perf && perf > 0) perfStatus = ['positive', RedIndicatorIcon]
  else if (perf && perf < 0) perfStatus = ['negative', GreenIndicatorIcon]
  return (
    <div className="fpi">
      <div className="fpi-left">
        <div className="fpi-content">
          <div className="fpi-content-perf">
            <div className="fpi-content-perf-result card-result">
              <div className="icon-line">
                <StyledIcon
                  className="fpi-content-icon perf-icon"
                  icon={PileIcon}
                  size={35}
                />
                <div>
                  <span className="euro-value">{displayedValue}</span>
                  <span className="card-indicator"> €</span>
                </div>
              </div>
            </div>
            {perfStatus[0] === 'positive' ||
            perfStatus[0] === 'negative' ||
            perfStatus[0] === 'zero' ? (
              <div className="fpi-content-perf-indicator bilan-card card-text">
                <div className="icon-line">
                  <StyledIcon
                    className="fpi-content-icon perf-icon"
                    icon={perfStatus[1]}
                    size={35}
                  />
                  <div className="evolution-text">
                    {t('performance_indicator.bilan.text1')}
                    {analysisDate &&
                      convertDateToMonthString(
                        analysisDate.plus({ month: -2 })
                      ).substring(3)}{' '}
                    :
                    <span
                      className={`fpi-content-perf-indicator-kpi ${perfStatus[0]} card-text-bold`}
                    >
                      {perfStatus[0] === 'positive' ? ' + ' : ' '}
                      {perfString}%
                    </span>
                    <br />
                    <span>
                      {t('performance_indicator.bilan.text2')}
                      <span className="diff-value">{diffString} €</span>
                    </span>
                  </div>
                </div>
              </div>
            ) : (
              <div className="fpi-content-perf-indicator card-text error">
                <StyledIcon
                  className="fpi-content-icon perf-icon"
                  icon={perfStatus[1]}
                  size={35}
                />

                {fluidLackOfData.length !== 0 ? (
                  <div>
                    <div>
                      {' '}
                      {t('performance_indicator.error_no_compare_no_data')}{' '}
                    </div>
                    <div>
                      {fluidLackOfData.map(fluidType => {
                        return (
                          <div key={fluidType} className="fluid-enum">
                            {' '}
                            - {t(
                              'FLUID.' + FluidType[fluidType] + '.NAME'
                            )}{' '}
                          </div>
                        )
                      })}
                    </div>
                  </div>
                ) : (
                  <div>
                    <div> {t('performance_indicator.error_no_compare')} </div>
                    <div>
                      {' '}
                      {t('performance_indicator.error_no_compare_reason')}{' '}
                    </div>
                  </div>
                )}
              </div>
            )}
          </div>
        </div>
      </div>
    </div>
  )
}

export default PerformanceIndicatorContent
