import { Client, Q, QueryDefinition, QueryResult } from 'cozy-client'
import { PROFILE_DOCTYPE } from 'doctypes'
import { DateTime } from 'luxon'
import { Profile, ProfileEntity } from 'models'

export default class ProfileService {
  private readonly _client: Client

  constructor(_client: Client) {
    this._client = _client
  }

  private getDate(date: string): DateTime {
    return DateTime.fromISO(date, {
      zone: 'utc',
    })
  }

  /**
   * Retrieve Profile from the ProfileEntity
   * @param {ProfileEntity} profileEntity
   * @returns {Profile}
   */
  private parseProfileEntityToProfile(profileEntity: ProfileEntity): Profile {
    const yesterday: DateTime = DateTime.local()
      .minus({ day: 1 })
      .startOf('day')
    const partnersIssueSeenDate = {
      enedis: yesterday,
      egl: yesterday,
      grdf: yesterday,
    }

    const profile: Profile = {
      ...profileEntity,
      monthlyAnalysisDate:
        typeof profileEntity.monthlyAnalysisDate === 'string'
          ? this.getDate(profileEntity.monthlyAnalysisDate)
          : profileEntity.monthlyAnalysisDate,
      lastConnectionDate: this.getDate(profileEntity.lastConnectionDate),
      partnersIssueSeenDate: partnersIssueSeenDate,
      activateHalfHourDate: this.getDate(profileEntity.activateHalfHourDate),
      customPopupDate: this.getDate(profileEntity.customPopupDate),
    }
    return profile
  }

  public async getProfile(): Promise<Profile | null> {
    const query: QueryDefinition = Q(PROFILE_DOCTYPE)
    const {
      data: [profile],
    }: QueryResult<ProfileEntity[]> = await this._client.query(query.limitBy(1))
    const profileEntity: ProfileEntity | null = profile ? profile : null
    if (profileEntity) {
      return this.parseProfileEntityToProfile(profileEntity)
    }
    return null
  }

  public async updateProfile(
    attributes: Partial<Profile>
  ): Promise<Profile | null> {
    const query: QueryDefinition = Q(PROFILE_DOCTYPE)
    const {
      data: [doc],
    }: QueryResult<ProfileEntity[]> = await this._client.query(query.limitBy(1))
    const { data: profileEntity }: QueryResult<ProfileEntity | null> =
      await this._client.save({
        ...doc,
        ...attributes,
      })
    if (profileEntity) {
      return this.parseProfileEntityToProfile(profileEntity)
    }
    return null
  }
}
