import { DateTime } from 'luxon'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { TimePeriod } from 'models'
import ConfigService from 'services/fluidConfig.service'

export default class TimePeriodService {
  public getTimePeriods(
    date: DateTime,
    fluidTypes: FluidType[],
    timeStep: TimeStep
  ): {
    timePeriod: TimePeriod
    comparisonTimePeriod: TimePeriod
  } {
    const endOfOffsetDate = this.applyOffsetToDate(
      date,
      this.calculateFluidTypesOffset(fluidTypes)
    ).endOf('day')

    let lastDayOfCompletePeriod

    if (this.isLastDayOfTimePeriod(endOfOffsetDate, timeStep))
      lastDayOfCompletePeriod = endOfOffsetDate
    else
      lastDayOfCompletePeriod = this.getLastDayOfCompletePeriod(
        endOfOffsetDate,
        timeStep
      ).endOf('day')

    const timePeriod: TimePeriod = this.getLastCompletePeriod(
      lastDayOfCompletePeriod,
      timeStep
    )
    const comparisonTimePeriod: TimePeriod = this.getComparisonTimePeriod(
      timePeriod,
      timeStep
    )

    return {
      timePeriod: timePeriod,
      comparisonTimePeriod: comparisonTimePeriod,
    }
  }

  public calculateFluidTypesOffset(fluidTypes: FluidType[]): number {
    const allOffsets: number[] = []
    const configService = new ConfigService()
    const fluidConfig = configService.getFluidConfig()
    fluidTypes.forEach(fluidType =>
      allOffsets.push(fluidConfig[fluidType].dataDelayOffset)
    )

    // eslint-disable-next-line prefer-spread
    const maxOffset = Math.max.apply(Math, allOffsets)
    return maxOffset
  }

  public applyOffsetToDate(date: DateTime, offset: number): DateTime {
    return date
      .setZone('utc', {
        keepLocalTime: true,
      })
      .minus({ days: offset })
  }

  public isLastDayOfTimePeriod(date: DateTime, timeStep: TimeStep) {
    const lastDayOfTimePeriod = this.getLastDayOfTimePeriod(date, timeStep)

    return (
      date.year === lastDayOfTimePeriod.year &&
      date.month === lastDayOfTimePeriod.month &&
      date.day === lastDayOfTimePeriod.day
    )
  }

  public getLastDayOfTimePeriod(date: DateTime, timeStep: TimeStep): DateTime {
    switch (timeStep) {
      case TimeStep.HALF_AN_HOUR:
        return DateTime.local(date.year, date.month, date.day).setZone('utc', {
          keepLocalTime: true,
        })
      case TimeStep.WEEK:
        return DateTime.local(date.year, date.month, date.day)
          .setZone('utc', {
            keepLocalTime: true,
          })
          .endOf('week')
      case TimeStep.DAY:
        return DateTime.local(date.year, date.month, date.day)
          .setZone('utc', {
            keepLocalTime: true,
          })
          .endOf('month')
      case TimeStep.MONTH:
        return DateTime.local(date.year, date.month, 1)
          .setZone('utc', {
            keepLocalTime: true,
          })
          .plus({ months: 1 })
          .minus({ days: 1 })
      case TimeStep.YEAR:
        return DateTime.local(date.year, 1, 1)
          .setZone('utc', {
            keepLocalTime: true,
          })
          .plus({ years: 1 })
          .minus({ days: 1 })
      default:
        throw new Error('TimeStep unknown')
    }
  }

  public getLastDayOfCompletePeriod(date: DateTime, timeStep: TimeStep) {
    switch (timeStep) {
      case TimeStep.HALF_AN_HOUR:
        return DateTime.local(date.year, date.month, date.day).setZone('utc', {
          keepLocalTime: true,
        })
      case TimeStep.WEEK:
        return DateTime.local(date.year, date.month, date.day)
          .setZone('utc', {
            keepLocalTime: true,
          })
          .startOf('week')
          .minus({ days: 1 })
      case TimeStep.DAY:
        return DateTime.local(date.year, date.month, date.day)
          .setZone('utc', {
            keepLocalTime: true,
          })
          .startOf('month')
          .minus({ days: 1 })
      case TimeStep.MONTH:
        return DateTime.local(date.year, date.month, 1)
          .setZone('utc', {
            keepLocalTime: true,
          })
          .minus({
            days: 1,
          })
      case TimeStep.YEAR:
        return DateTime.local(date.year, 1, 1)
          .setZone('utc', {
            keepLocalTime: true,
          })
          .minus({ days: 1 })
      default:
        throw new Error('TimeStep unknown')
    }
  }

  public getStartDateFromEndDateByTimeStep(
    startDate: DateTime,
    timeStep: TimeStep
  ) {
    switch (timeStep) {
      case TimeStep.HALF_AN_HOUR:
        return DateTime.local(
          startDate.year,
          startDate.month,
          startDate.day
        ).setZone('utc', {
          keepLocalTime: true,
        })
      case TimeStep.WEEK:
        return DateTime.local(startDate.year, startDate.month, startDate.day)
          .setZone('utc', {
            keepLocalTime: true,
          })
          .startOf('week')
      case TimeStep.DAY:
        return DateTime.local(startDate.year, startDate.month, 1)
          .setZone('utc', {
            keepLocalTime: true,
          })
          .startOf('month')
      case TimeStep.MONTH:
        return DateTime.local(startDate.year, startDate.month, 1).setZone(
          'utc',
          {
            keepLocalTime: true,
          }
        )
      case TimeStep.YEAR:
        return DateTime.local(startDate.year, 1, 1).setZone('utc', {
          keepLocalTime: true,
        })
      default:
        throw new Error('TimeStep unknown')
    }
  }

  public getLastCompletePeriod(
    lastDay: DateTime,
    timeStep: TimeStep
  ): TimePeriod {
    // calculate last day of the tobe coimpleted period
    const lastCompleteTimePeriod = {
      startDate: this.getStartDateFromEndDateByTimeStep(lastDay, timeStep),
      endDate: lastDay,
    }
    return lastCompleteTimePeriod
  }

  public getComparisonTimePeriod(
    timePeriod: TimePeriod,
    timeStep: TimeStep
  ): TimePeriod {
    let comparisonTimePeriodStartDate: DateTime
    let comparisonTimePeriodEndDate: DateTime

    switch (timeStep) {
      case TimeStep.HALF_AN_HOUR:
        comparisonTimePeriodStartDate = timePeriod.startDate.minus({ days: 1 })
        comparisonTimePeriodEndDate = timePeriod.endDate.minus({
          days: 1,
        })
        break
      case TimeStep.WEEK:
        comparisonTimePeriodStartDate = timePeriod.startDate.minus({ week: 1 })
        comparisonTimePeriodEndDate = timePeriod.endDate.minus({
          week: 1,
        })
        break
      case TimeStep.DAY:
        comparisonTimePeriodStartDate = timePeriod.startDate
          .setZone('utc', {
            keepLocalTime: true,
          })
          .minus({ days: 1 })
          .startOf('month')
        comparisonTimePeriodEndDate = timePeriod.startDate.minus({ days: 1 })
        break
      case TimeStep.MONTH:
        comparisonTimePeriodStartDate = DateTime.local(
          timePeriod.startDate.year,
          timePeriod.startDate.month - 1,
          timePeriod.startDate.day
        ).setZone('utc', {
          keepLocalTime: true,
        })
        comparisonTimePeriodEndDate = DateTime.local(
          timePeriod.endDate.year,
          timePeriod.endDate.month - 1,
          1
        )
          .setZone('utc', {
            keepLocalTime: true,
          })
          .endOf('month')
        break
      case TimeStep.YEAR:
        comparisonTimePeriodStartDate = DateTime.local(
          timePeriod.startDate.year - 1,
          timePeriod.startDate.month,
          timePeriod.startDate.day
        ).setZone('utc', {
          keepLocalTime: true,
        })
        comparisonTimePeriodEndDate = DateTime.local(
          timePeriod.endDate.year - 1,
          timePeriod.endDate.month,
          timePeriod.endDate.day
        )
          .setZone('utc', {
            keepLocalTime: true,
          })
          .endOf('year')
        break
      default:
        throw new Error('TimeStep unknown')
    }

    const comparisonTimePeriod: TimePeriod = {
      startDate: comparisonTimePeriodStartDate,
      endDate: comparisonTimePeriodEndDate,
    }
    return comparisonTimePeriod
  }
}
